// RTTYGen.cpp: implementation of the CRTTYGen class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "wscgen.h"
#include "RTTYGen.h"
#include "math.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define RMSCONSTANT 1.0
#define SYMBOL_RATE 45.454545454545454545 //45.45 Symbols per Second
#define FREQ_SHIFT 170.0 //170 Hz	

#define RUS 0
#define LAT 31
#define FIG 27

#define SPACE 4
#define CR 2
#define LF 8

LPCSTR CRTTYGen::m_pszText="RYRYRYRYRYRYRYRY  \n"\
                           "01234567890\n"\
                           "ABCDEFGHIJKLMNOPQRSTUVWXYZ\n"\
                           "\n"\
                           "iiiiiiiiiiiiiiiiii";

static char EncoderLAT[]=" T\rO HNM\nLRGIPCVEZDBSYFXAWJdUQKl";  //English
static char EncoderFIG[]=" 5\r9 #,.\n)4&80:=3+$?'6!/-2 d71(l";  //Figures Case (Eng)
static char EncoderRUS[]=" \r \ndl";  //Russian



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRTTYGen::CRTTYGen()
{
	m_2PI = 8.0 * atan(1.0);		//make constant 2 PI

}

CRTTYGen::~CRTTYGen()
{

}

/////////////////////////////////////////////////////////////
//Initialize the RTTY generator with the sample frequncy Fs,
// the carrier frequency freq.
/////////////////////////////////////////////////////////////
void CRTTYGen::InitRTTY(INT Fs, double freq)
{
	SetRTTYFreq( Fs, freq,0,0,FALSE,FALSE);	//set carrier frequency
	m_RTTYSecPerSamp = 1.0/Fs;
	m_RTTYPeriodUpdate = 1.0/SYMBOL_RATE;
	m_RTTYTime = 0;
   m_RTTYUpdateTime=0;
   m_nShift=0;
   m_bUSOS=TRUE;
   m_pChar=m_pszText;
   for(int i=0;i<32;i++)
   {
      m_Encoder[EncoderRUS[i]]=MAKEWORD(i,RUS);
      m_Encoder[EncoderFIG[i]]=MAKEWORD(i,FIG);
      m_Encoder[EncoderLAT[i]]=MAKEWORD(i,LAT);
   }
   m_Encoder[' ']=SPACE;  //Shift is not needed for the following characters
   m_Encoder['\0']=0;
   m_Encoder['\r']=CR;
   m_Encoder['\n']=LF;
   m_Encoder['d']=FIG;  //Figues shift
   m_Encoder['l']=LAT;  //Latin shift
   m_Encoder['r']=RUS;   //Russian (third) shift
}

/////////////////////////////////////////////////////////////
// Call to change the psk31 carrier frequency
/////////////////////////////////////////////////////////////
void CRTTYGen::SetRTTYFreq(INT Fs, double freq, INT rate, INT range, BOOL idle, BOOL rev)
{
	m_RTTYPhaseIncH = m_2PI * freq/(double)Fs;		//carrier frequency MARK
	m_RTTYPhaseIncL = m_2PI * freq/(double)Fs;		//carrier frequency SPACE
   if(rev)
   {
	   m_RTTYPhaseIncH -= (m_2PI * FREQ_SHIFT/(double)Fs); //Correct MARK (reverse)
   }
   else
   {
      m_RTTYPhaseIncL -= (m_2PI * FREQ_SHIFT/(double)Fs); //Correct SPACE
   }
   m_bIdle=idle;
}


void CRTTYGen::EmitChar(BYTE bits)
{
   m_Bits.AddTail(STARTBIT);
   for(int i=0;i<5;i++)
   {
      if(bits & 0x10)m_Bits.AddTail(MARKBIT);
      else m_Bits.AddTail(SPACEBIT);
      bits<<=1;
   }
   m_Bits.AddTail(STOPBIT);
}

void CRTTYGen::FillBuffer()
{
   if(m_bIdle)
   {
      EmitChar(LAT);//Idle signal LAT LAT LAT .....
   }
   else
   {
      m_pChar++;
      if(*m_pChar==0)m_pChar=m_pszText;
      WORD bits;
      if(!m_Encoder.Lookup(*m_pChar,bits))bits=LAT; //Send IDLE signal for invalid characters
      BYTE shift=HIBYTE(bits);
      if(shift!=0 && shift!=m_nShift)
      {
         m_nShift=shift;
         EmitChar(shift);
      }
      if(bits==SPACE && m_bUSOS)m_nShift=LAT;
      EmitChar((BYTE)bits);
   }
}

BOOL CRTTYGen::GetNextSymbol()
{
   if(m_Bits.IsEmpty())FillBuffer();
   BYTE bit=m_Bits.RemoveHead();
   m_RTTYUpdateTime=m_RTTYPeriodUpdate; //Start and Data bits
   if(bit==STOPBIT)m_RTTYUpdateTime*=1.5;
   return (bit & MARKBIT) != 0;
}

/////////////////////////////////////////////////////////////
// generates n samples of RTTY waveform in pData with average RMS value of 1.0
/////////////////////////////////////////////////////////////
BOOL CRTTYGen::CalcRTTY(double *pData, INT n)
{
static double tc = 0.0;
static double p=m_RTTYPhaseIncH;
	for(int i=0; i<n; i++ )
	{
      m_RTTYTime+=m_RTTYSecPerSamp;
		if( m_RTTYTime >= m_RTTYUpdateTime)
		{
         if(GetNextSymbol())
         {
            p=m_RTTYPhaseIncH;
         }
         else
         {
            p=m_RTTYPhaseIncL;
         }
         m_RTTYTime=0.0;
		}
	   tc = tc + p;	// increment radian phase count
      tc = fmod(tc,m_2PI);			//keep radian counter bounded
		pData[i] = 1.41*sin( tc );
	}
	return FALSE;
}
